// David Pilling 1st February 2016
// HC-SR04 with STM32 using capture register


#include <stdarg.h>

void pr(char *fmt, ... ){
        char buf[128]; // resulting string limited to 128 chars
        va_list args;
        va_start (args, fmt );
        vsnprintf(buf, 128, fmt, args);
        va_end (args);
        Serial.println(buf);
}


#define pinLED PC13
#define pinEcho PB6
#define pinTrigger PA7


// copied from http://forums.leaflabs.com/topic.php?id=1038#post-6720

void setuptimer(void)
{
 TIMER4_BASE->CCMR1 = 0x00F1;    //channel 1 configure as input, IC1 mapped on TI1
 TIMER4_BASE->CCMR2 = 0x0000;    //channels 3&4 disabled
 TIMER4_BASE->CCER = 0x0001;   //configure channel 1 as an input
 TIMER4_BASE->SMCR = 0x0000;   //all slave modes disabled, CC transitions on clock
 TIMER4_BASE->CR1 = 0x0001;    //enable the counter
 TIMER4_BASE->CR2 = 0x0000;    //connect TIM4_CH1 pin to TI1 input
 TIMER4_BASE->PSC = 36;    // set prescaler to give 2 MHz 

 timer_attach_interrupt(TIMER4, TIMER_CC1_INTERRUPT, timerinterrupt);

 timer_resume(TIMER4);
}

volatile unsigned int risetime;
volatile unsigned int duration;

void timerinterrupt(void)
{
  //get the bit to see if this is an up or down pulse
 int  polarity = TIMER4_BASE->CCER & 0x0002;

 if(polarity == 0) //this is a rising edge capture
 {
  risetime = TIMER4_BASE->CCR1;   //clear the interrupt by reading CCR1 value
  TIMER4_BASE->CCER |= 0x0002;    //change polarity to capture falling edge
 }
 else  //this is a falling edge capture
 {
  unsigned int value = TIMER4_BASE->CCR1;    //read the captured value
  duration=(value-risetime)/2;
  TIMER4_BASE->CCER &= ~0x0002;   //change polarity to capture rising edge
 }
}




void setup() 
{
 Serial.begin(9600);
 pinMode(pinLED, OUTPUT);
 pinMode(pinEcho, INPUT);
 pinMode(pinTrigger, OUTPUT);

 setuptimer();
}



void loop() 
{
 digitalWrite(pinLED, HIGH);
 delay(100);
 digitalWrite(pinLED, LOW);
 delay(100);

 digitalWrite(pinTrigger, 1);   
 delayMicroseconds(10); 
 digitalWrite(pinTrigger, 0); 

 unsigned long timeout=millis()+100;
 duration=0;
 
 while(1)
 {
  if(duration) break;
  if(millis()>timeout) break;
 }

 pr("time %lu",duration);
}

